# files management class
from os import listdir
from os.path import isfile, join
import os
import class_data_encrypt
import json
import class_ftp_util
import class_ia_configure

class init():

    def __init__(self, filePath):
        self.filePath = filePath
        self.clsENCRYPT = class_data_encrypt.init('SeaDeep2021@WSX')
        self.clsCFG = class_ia_configure.init('./ia.ini')
        self.cfgData = self.clsCFG.parseCFGData()

    def getFileLists(self):
        allFileLists = [f for f in listdir(self.filePath) if isfile(join(self.filePath, f))]
        return allFileLists

    def removeFile(self, fileName):
        os.remove(self.filePath + '/' + fileName)
        return

    def writeFileBytesData(self, fileName, rawData):
        with open(self.filePath + '/' + fileName, 'wb') as writer:
            writer.write(rawData)
        return

    def readFileBytesData(self, fileName):
        with open(self.filePath + '/' + fileName, 'rb') as reader:
            rawData = reader.read()
        return rawData

    def parseModelDownloadData(self, payloadMsg):
        payloadMsgDecode = json.loads(self.clsENCRYPT.dataDecode(payloadMsg))
        print('message decoded=======>', payloadMsgDecode)        
        downloadProfile={
            'ia_id': payloadMsgDecode['ia_id'], 
            'ai_model_id': payloadMsgDecode['ai_model_id'],
            'mqtt_type': payloadMsgDecode['mqtt_type'], 
            'class_name': payloadMsgDecode['class_name'], 
            'deploy_datetime': payloadMsgDecode['deploy_datetime'], 
            'ftp_ip': payloadMsgDecode['ftp_ip'], 
            'ftp_port': int(payloadMsgDecode['ftp_port']), 
            'ftp_username': payloadMsgDecode['ftp_username'], 
            'ftp_password': payloadMsgDecode['ftp_password'], 
            'model_weight_path': payloadMsgDecode['model_weight_path'], 
            'model_factors_path': payloadMsgDecode['model_factors_path'], 
            'inf_class_path': payloadMsgDecode['inf_class_path']
        }        
        return downloadProfile

    def downLoadAllModelFiles(self):
        fileLists = self.getFileLists()
        #print(fileLists)
        for i in range(0, len(fileLists)):
            encodedMsg = self.readFileBytesData(fileLists[i])
            fileData = self.parseModelDownloadData(encodedMsg)
            #print('file data=========>', fileData)
            mqttType = fileData['mqtt_type']
            if mqttType=='ia_model_deploy':
                clsFTP = class_ftp_util.init(fileData['ftp_ip'], fileData['ftp_username'], fileData['ftp_password'])
                print('ftp===========>done!')
                iaID = fileData['ia_id']
                aiModelID = fileData['ai_model_id']
                className = fileData['class_name']
                deployDateTime = fileData['deploy_datetime']

                #weightFile = aiModelID + '-best_model.pth'
                #factorFile = aiModelID + '-factor.json'
                #infClassFile = aiModelID + '-inf-class.txt'

                remoteWeightFilePath = fileData['model_weight_path']
                remoteFactorFilePath = fileData['model_factors_path']
                remoteInfClassFilePath = fileData['inf_class_path']

                weightFile = aiModelID + '-' + remoteWeightFilePath.split('/')[-1:][0]
                factorFile = aiModelID + '-' + remoteFactorFilePath.split('/')[-1:][0]
                infClassFile = aiModelID + '-' + remoteInfClassFilePath.split('/')[-1:][0]

                localWeightFilePath = '.' + self.cfgData['model_weight_path'] + '/' + weightFile
                localFactorFilePath = '.' + self.cfgData['model_factor_path'] + '/' + factorFile
                localInfClassFilePath = '.' + self.cfgData['model_inf_class_path'] + '/' + infClassFile

                clsFTP.get_files(remoteWeightFilePath, localWeightFilePath)
                clsFTP.get_files(remoteFactorFilePath, localFactorFilePath)
                clsFTP.get_files(remoteInfClassFilePath, localInfClassFilePath)
                self.clsCFG.updateModelTaskProfile(iaID, aiModelID, className, weightFile, factorFile, infClassFile, deployDateTime)     

                iaIDLists = self.clsCFG.parseCFGData()['ia_uuid'].split(',')
                iaStatusLists = self.clsCFG.parseCFGData()['ia_status'].split(',')
                iaNewStatusString = self.clsCFG.updateStringToken(iaIDLists, iaID, iaStatusLists, 'deployed')
                print('========>', iaNewStatusString)
                self.clsCFG.writeData('IA PROFILE', 'STATUS', iaNewStatusString)
                self.removeFile(fileLists[i])

            else:
                print('file is not the model deployment request...')
        return